/*
===============================================================================

  FILE:  SVreader_sms.h
  
  CONTENTS:
  
    Reads a mesh from the SMS format (scorec format) and presents it as a (poor)
    streaming mesh. Only the vertex positions and the tetrahedral connectivity
    are extracted from the sms file. Other fields are ignored.
  
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2007 martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    4 September 2008 -- created after the ranch trip to see kaoru and her pony

===============================================================================
*/
#ifndef SVREADER_SMS_H
#define SVREADER_SMS_H

#include "svreader.h"

#include <stdio.h>

class SVreader_sms : public SVreader
{
public:

  // SVreader interface function implementations

  SVevent read_element();
  SVevent read_event();

  void close(bool close_file=true);

  // SVreader_sms functions

  bool open(FILE* file);

  SVreader_sms();
  ~SVreader_sms();

private:
  int version;
  I32 nedges;
  I32 nfaces;
  I32 npoints;
  FILE* file;
  char* line;
  I32* faces;
};

#endif
