/*
===============================================================================

  FILE:  SVwriteBuffered.h
  
  CONTENTS:
  
    Writes a Streaming Volume Mesh with a "little-cache" aware greedy reordering
    of tetrahedra that is subject to a constraint of maximal delay of tetrahedra.

  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2005  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    17 April 2005 -- adapted from functionality of the similar SMC reorderer 
  
===============================================================================
*/
#ifndef SVWRITE_BUFFERED_H
#define SVWRITE_BUFFERED_H

#include "svwriter.h"

class SVwriteBuffered : public SVwriter
{
public:
  // svwriter interface function implementations

  void add_comment(const char* comment);

  void set_nverts(I32 nverts);
  void set_ncells(I32 ncells);
  void set_v_pnum_f(I32 v_pnum_f);
  void set_v_pbox_f(const float* v_pmin_f, const float* v_pmax_f);
  void set_c_pnum_f(I32 c_pnum_f);
  void set_c_pbox_f(const float* c_pmin_f, const float* c_pmax_f);

  void write_vertex(const float* v_data_f);

  void write_tetrahedron(const I32* c_idx, const bool* c_final=0);
  void write_tetrahedron(const I32* c_idx, const float* c_data_f, const bool* c_final=0);

  void write_hexahedron(const I32* c_idx, const bool* c_final=0);
  void write_hexahedron(const I32* c_idx, const float* c_data_f, const bool* c_final=0);

  void write_finalized(I32 final_idx);

  void close(bool close_file=true, bool update_buffer=false);

  // svwritebuffered functions

  bool open(SVwriter* svwriter, int max_delay=100);

  SVwriteBuffered();
  ~SVwriteBuffered();

private:
  SVwriter* svwriter;
  int max_delay;

  void write_tetrahedron_delayed();
};

#endif
