/*
===============================================================================

  FILE:  littlecache.h

  CONTENTS:


  PROGRAMMERS:

    martin isenburg@cs.unc.edu

  COPYRIGHT:

    copyright (C) 2003  martin isenburg@cs.unc.edu

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  CHANGE HISTORY:

    07 October 2003 -- initial version created the day of the California recall

===============================================================================
*/
#ifndef LITTLE_CACHE_H
#define LITTLE_CACHE_H

#include <assert.h>

class LittleCache
{
public:
  LittleCache(int size = 3);
  ~LittleCache();

  inline void put(void* d0, void* d1, void* d2);
  inline void put(int i0, int i1, int i2, void* d0, void* d1, void* d2);
  inline void put(void* d0, void* d1, void* d2, void* d3, void* d4, void* d5);
  inline void put(void* d0, void* d1, void* d2, void* d3, void* d4, void* d5, void* d6, void* d7);
  inline void put(void* d0, void* d1, void* d2, void* d3);
  inline void* get(int i);
  inline int pos(int i);
  inline int pos(void* d);

private:
  int size;
  int* index;
  void** data;
};

inline LittleCache::LittleCache(int size)
{
  this->size = size;
  index = new int[size];
  data = new void*[size];
	for (int i=0; i<size; i++)
	{
    index[i] = -1;
    data[i] = 0;
  }
}

inline LittleCache::~LittleCache()
{
  delete [] index;
  delete [] data;
}

inline void LittleCache::put(void* d0, void* d1, void* d2)
{
  assert(size >= 3);
  data[0] = d0;
  data[1] = d1;
  data[2] = d2;
}

inline void LittleCache::put(int i0, int i1, int i2, void* d0, void* d1, void* d2)
{
  assert(size >= 3);
  index[0] = i0;
  index[1] = i1;
  index[2] = i2;
  data[0] = d0;
  data[1] = d1;
  data[2] = d2;
}

inline void LittleCache::put(void* d0, void* d1, void* d2, void* d3)
{
  assert(size >= 4);
  data[0] = d0;
  data[1] = d1;
  data[2] = d2;
  data[3] = d3;
}

inline void LittleCache::put(void* d0, void* d1, void* d2, void* d3, void* d4, void* d5)
{
  assert(size >= 6);
	data[0] = d0;
	data[1] = d1;
	data[2] = d2;
	data[3] = d3;
	data[4] = d4;
	data[5] = d5;
}

inline void LittleCache::put(void* d0, void* d1, void* d2, void* d3, void* d4, void* d5, void* d6, void* d7)
{
  assert(size >= 8);
	data[0] = d0;
	data[1] = d1;
	data[2] = d2;
	data[3] = d3;
	data[4] = d4;
	data[5] = d5;
	data[6] = d6;
	data[7] = d7;
}

inline void* LittleCache::get(int i)
{
  assert(size > i);
  return data[i];
}

inline int LittleCache::pos(int i)
{
  assert(size > i);
  for (int j = 0; j < size; j++)
  {
    if (i == index[j])
    {
      return j;
    }
  }
  return -1;
}

inline int LittleCache::pos(void* d)
{
  for (int j = 0; j < size; j++)
  {
    if (d == data[j])
    {
      return j;
    }
  }
  return -1;
}

#endif
