/*
===============================================================================

  FILE:  SVreader_sms.cpp
  
  CONTENTS:
  
    see corresponding header file
  
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2008 martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    see corresponding header file
  
===============================================================================
*/
#include "svreader_sms.h"

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>

#include "vecnfv.h"
#include "vecniv.h"

#define MAX_LINE_SIZE 1024

bool SVreader_sms::open(FILE* file)
{
  if (file == 0)
  {
    return false;
  }
  this->file = file;

  type = SV_TET;

  post_order = false;

  line = (char*)malloc(sizeof(char)*MAX_LINE_SIZE);

  fscanf(file,"%s %d",line,&version);
  
  if (version == 3)
  {
    int pid, total, numPEs, totalCBEntities;
    fscanf(file,"%d %d %d %d",&total, &pid, &numPEs, &totalCBEntities);

    int peid, pedim, peowner, penumBPs;
    for (int i=0; i<numPEs;++i)
    {
      fscanf(file,"%d %d %d %d",&peid, &pedim, &peowner, &penumBPs);
      int bp;
      for (int j=0; j<penumBPs;++j)
      {
        fscanf(file,"%d",&bp);
      }
    }
  }
  
  fscanf(file,"%d %d %d %d %d",&ncells,&nfaces,&nedges, &nverts, &npoints);

  v_count = 0;
  c_count = 0;

  v_pnum_f = 3;
  v_prop_f = new float[v_pnum_f];

  c_nverts = 4;
  c_idx = new I32[8]; // only TET meshes are supported
  c_final = new bool[8];

  c_final[0] = false; 
  c_final[1] = false;
  c_final[2] = false;
  c_final[3] = false;
  c_final[4] = false; 
  c_final[5] = false;
  c_final[6] = false;
  c_final[7] = false;

  return true;
}

void SVreader_sms::close(bool close_file)
{
  // close of SVreader_sms
  if (close_file && file && file != stdin) fclose(file);
  file = 0;
  if (line)
  {
    free(line);
    line = 0;
  }
  if (faces)
  {
    delete [] faces;
    faces = 0;
  }
  // close of SVreader interface
  v_count = -1;
  c_count = -1;
  if (v_prop_f) delete [] v_prop_f; v_prop_f = 0;
  if (c_idx) delete [] c_idx; c_idx = 0;
  if (c_final) delete [] c_final; c_final = 0;
}

SVevent SVreader_sms::read_element()
{
  if (line == 0)
  {
    return SV_EOF;
  }

  if (v_count < nverts)
  {
    v_idx = v_count+1;
    if (version == 3)
    {
      int vid;
      fscanf(file,"%d",&vid);
    }
    int GEntityId;
    fscanf(file,"%d",&GEntityId);
    if (GEntityId)
    {
      int GEntityType;
      int EntityNbConnections;
      fscanf(file,"%d %d %f %f %f",&GEntityType,&EntityNbConnections,&(v_prop_f[0]), &(v_prop_f[1]), &(v_prop_f[2]));
      float u,v;
      int patch;
      switch(GEntityType)
      {
      case 0:
        break;
      case 1:
	      fscanf(file,"%f",&u);
	      break;
      case 2:
	      fscanf(file,"%f %f %d",&u,&v,&patch);
	      break;
      case 3:
	      break;
      default:
	      fprintf(stderr, "WARNING: unknown entity type");
	      exit(1);
      }
      if (version == 3)
      {
	      int pclassid;
	      int ucbid;
	      fscanf(file,"%d %d", &pclassid, &ucbid);
      }
    }
    v_count++;
    return SV_VERTEX;
  }
  else if (c_count < ncells)
  {
    if (c_count == 0)  // if we just start reading the cell block
    {
      int i;
      I32* edges = new I32[2*nedges];
      // skip the edges
      for(i=0;i<nedges;i++)
      {
	      int GEntityId;
	      fscanf(file,"%d",&GEntityId);
	      if (GEntityId)
	      {
	        int GEntityType;
	        int VertexId1;
	        int VertexId2;
	        int EntityNbConnections;
	        int nbPts;
	        fscanf(file,"%d %d %d %d %d", &GEntityType, &VertexId1, &VertexId2, &EntityNbConnections, &nbPts);
	        edges[2*i] = VertexId1;
	        edges[2*i+1] = VertexId2;
	        float u,v;
	        int patch;
	        for(int j=0;j<nbPts;j++)
	        {
	          switch(GEntityType)
	          {
	          case 0:
	            break;
	          case 1: fscanf(file,"%f",&u);
	            break;
	          case 2: fscanf(file,"%f %f %d",&u,&v,&patch);
	            break;
	          case 3:
	            break;
	          default:
	            fprintf(stderr, "WARNING: unknown entity type\n");
	            exit(1);
	          }
	        }
	        if (version == 3)
	        {
	          int pclassid;
	          int ucbid;
	          fscanf(file,"%d %d", &pclassid, &ucbid);
	        }
	      }
      }  // end of skipping edges

      faces = new I32[4*nfaces];

      // skip the faces
      for(i=0;i<nfaces;i++)
      {
	      int GEntityId;
	      fscanf(file,"%d",&GEntityId);
	      if(GEntityId)
	      {
	        int GEntityType;
	        int NbEdgesOnFace;
	        fscanf(file,"%d %d",&GEntityType, &NbEdgesOnFace);
	        int nbPts;
	        if (NbEdgesOnFace == 3)
	        {
	          int Edge1;
	          int Edge2;
	          int Edge3;
	          fscanf(file,"%d %d %d %d",&Edge1,&Edge2,&Edge3,&nbPts);
	          int iVertex1 = (Edge1>0)?edges[2*(abs(Edge1)-1)]:edges[2*(abs(Edge1)-1)+1];
	          int iVertex2 = (Edge2>0)?edges[2*(abs(Edge2)-1)]:edges[2*(abs(Edge2)-1)+1];
	          int iVertex3 = (Edge3>0)?edges[2*(abs(Edge3)-1)]:edges[2*(abs(Edge3)-1)+1];
	          faces[4*i] = iVertex1;
	          faces[4*i+1] = iVertex2;
	          faces[4*i+2] = iVertex3;
	        }
	        else if (NbEdgesOnFace == 4)
	        {
	          int Edge1;
	          int Edge2;
	          int Edge3;
	          int Edge4;
	          fscanf(file,"%d %d %d %d",&Edge1,&Edge2,&Edge3,&Edge4);
	          int iVertex1 = (Edge1>0)?edges[2*(abs(Edge1)-1)]:edges[2*(abs(Edge1)-1)+1];
	          int iVertex2 = (Edge2>0)?edges[2*(abs(Edge2)-1)]:edges[2*(abs(Edge2)-1)+1];
	          int iVertex3 = (Edge3>0)?edges[2*(abs(Edge3)-1)]:edges[2*(abs(Edge3)-1)+1];
	          int iVertex4 = (Edge4>0)?edges[2*(abs(Edge4)-1)]:edges[2*(abs(Edge4)-1)+1];
	          faces[4*i] = iVertex1;
	          faces[4*i+1] = iVertex2;
	          faces[4*i+2] = iVertex3;
	          faces[4*i+3] = iVertex4;
	        }
	        else
	        {
	          fprintf(stderr, "WARNING: wrong number %d of edges on face in svreader_sms\n", NbEdgesOnFace);
	          exit(1);
	        }
	        float u,v;
	        int patch;
	        for(int j=0;j<nbPts;j++)
	        {
	          switch(GEntityType)
	          {
	          case 0:
	            break;
	          case 1:
	            fscanf(file,"%f",&u);
	            break;
	          case 2:
	            fscanf(file,"%f %f %d",&u,&v,&patch);
	            break;
	          case 3:
	            break;
	          default:
	            fprintf(stderr, "WARNING: unknown entity type\n");
	            exit(1);
	          }
	        }
	        if (version == 3)
	        {
	          int pclassid;
	          int ucbid;
	          fscanf(file,"%d %d", &pclassid, &ucbid);
	        }
	      }
      } // end of skipping faces
      delete [] edges;
    }
    int GEntityId;
    fscanf(file,"%d",&GEntityId);
    if(GEntityId)
    {
      int NbFacesOnRegion;
      fscanf(file,"%d",&NbFacesOnRegion);
      int Dummy;
      if (NbFacesOnRegion == 4)
      {
	      I32 Face[4];
	      fscanf(file,"%d %d %d %d %d",&Face[0],&Face[1],&Face[2],&Face[3],&Dummy);
	      int iVertex1 = faces[4*(abs(Face[0])-1)];
	      int iVertex2 = faces[4*(abs(Face[0])-1)+1];
	      int iVertex3 = faces[4*(abs(Face[0])-1)+2];
	      int iVertex4 = -1;
              
	      int iVertex = faces[4*(abs(Face[1])-1)] ;
	      if(iVertex != iVertex1 && iVertex != iVertex2 && iVertex != iVertex3)
	      {
	        iVertex4 = iVertex;
	      }
	      iVertex = faces[4*(abs(Face[1])-1) + 1] ;
	      if(iVertex != iVertex1 && iVertex != iVertex2 && iVertex != iVertex3)
	      {
	        iVertex4 = iVertex;
	      }
        iVertex = faces[4*(abs(Face[1])-1) + 2] ;
	      if(iVertex != iVertex1 && iVertex != iVertex2 && iVertex != iVertex3)
	      {
	        iVertex4 = iVertex;
	      }
	      c_idx[0] = iVertex1;
	      c_idx[1] = iVertex2;
	      c_idx[2] = iVertex3;
	      c_idx[3] = iVertex4;
      } 
      else if (NbFacesOnRegion == 6)
      {
	      I32 Face[6];
	      fscanf(file,"%d %d %d %d %d %d %d",&Face[0],&Face[1],&Face[2], &Face[3], &Face[4], &Face[5], &Dummy);
// 	int iVertex1 = faces[4*(abs(Face[0])-1)];
// 	int iVertex2 = faces[4*(abs(Face[0])-1)+1];
// 	int iVertex3 = faces[4*(abs(Face[0])-1)+2];
// 	int iVertex4 = faces[4*(abs(Face[0])-1)+3];;
// 	int iVertex5 = faces[4*(abs(Face[5])-1)];
// 	int iVertex6 = faces[4*(abs(Face[5])-1)+1];
// 	int iVertex7 = faces[4*(abs(Face[5])-1)+2];
// 	int iVertex8 = faces[4*(abs(Face[5])-1)+3];
      }
      else
      {
	fprintf(stderr, "WARNING: wrong number %d of faces on region in svreader_sms", NbFacesOnRegion);
	exit(1);
      }
    }
    c_count++;
    c_nverts = 4;
    return SV_TETRAHEDRON;
  }
  free(line);
  line = 0;
  return SV_EOF;
}

SVevent SVreader_sms::read_event()
{
  return read_element();
}

SVreader_sms::SVreader_sms()
{
  // init of SVreader_sms
  file = 0;
  line = 0;
  faces = 0;
}

SVreader_sms::~SVreader_sms()
{
  // clean-up for SVreader_sms
  if (faces) delete [] faces;
}
