/*
===============================================================================

  FILE:  svwriter.cpp
  
  CONTENTS:

    see corresponding header file

  PROGRAMMERS:

    martin isenburg@cs.unc.edu

  COPYRIGHT:

    copyright (C) 2007  martin isenburg@cs.unc.edu

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  CHANGE HISTORY:

    see corresponding header file

===============================================================================
*/
#include "svwriter.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "vecnfv.h"

void SVwriter::add_comment(const char* comment)
{
  if (comments == 0)
  {
    ncomments = 0;
    comments = (char**)malloc(sizeof(char*)*10);
    comments[9] = (char*)-1;
  }
  else if (comments[ncomments] == (char*)-1)
  {
    comments = (char**)realloc(comments,sizeof(char*)*ncomments*2);
    comments[ncomments*2-1] = (char*)-1;
  }
  comments[ncomments] = strdup(comment);
  ncomments++;
}

void SVwriter::set_nverts(I32 nverts)
{
  this->nverts = nverts;
}

void SVwriter::set_ncells(I32 ncells)
{
  this->ncells = ncells;
}

void SVwriter::set_v_pnum_f(I32 v_pnum_f)
{
  this->v_pnum_f = v_pnum_f;
}

void SVwriter::set_v_pbox_f(const float* v_pmin_f, const float* v_pmax_f)
{
  if (v_pnum_f)
  {
    if (this->v_pmin_f == 0) delete [] this->v_pmin_f;
    if (this->v_pmax_f == 0) delete [] this->v_pmax_f;
    this->v_pmax_f = new float[v_pnum_f];
    this->v_pmin_f = new float[v_pnum_f];
    VecCopyNfv(this->v_pmin_f, v_pmin_f, v_pnum_f);
    VecCopyNfv(this->v_pmax_f, v_pmax_f, v_pnum_f);
  }
  else
  {
    fprintf(stderr, "WARNING: setting v_pbox_f with zero components\n");
  }
}

void SVwriter::set_c_pnum_f(I32 c_pnum_f)
{
  this->c_pnum_f = c_pnum_f;
}

void SVwriter::set_c_pbox_f(const float* c_pmin_f, const float* c_pmax_f)
{
  if (c_pnum_f)
  {
    if (this->c_pmin_f == 0) delete [] this->c_pmin_f;
    if (this->c_pmax_f == 0) delete [] this->c_pmax_f;
    this->c_pmax_f = new float[c_pnum_f];
    this->c_pmin_f = new float[c_pnum_f];
    VecCopyNfv(this->c_pmin_f, c_pmin_f, c_pnum_f);
    VecCopyNfv(this->c_pmax_f, c_pmax_f, c_pnum_f);
  }
  else
  {
    fprintf(stderr, "WARNING: setting c_bounding box_f with zero components\n");
    if (this->c_pmin_f) {delete [] this->c_pmin_f; this->c_pmin_f = 0;}
    if (this->c_pmax_f) {delete [] this->c_pmax_f; this->c_pmax_f = 0;}
  }
}

// void SVwriter::write_hexahedron(const int* c_idx, const bool* c_final)
// {
//   fprintf(stderr, "WARNING: this writer class does not implement the write_hexahedron() function\n");
// }
// 
// void SVwriter::write_hexahedron(const int* c_idx, const float* c_data_f, const bool* c_final)
// {
//   fprintf(stderr, "WARNING: this writer class does not implement the write_hexahedron() function\n");
// }

void SVwriter::close(bool close_file, bool update_header)
{
  fprintf(stderr, "WARNING: classes derived from SVwriter should implement their own close() function\n");
}

SVwriter::SVwriter()
{
  // init of SVwriter interface
  ncomments = 0;
  comments = 0;

  nverts = -1;
  ncells = -1;

  v_count = -1;
  c_count = -1;

  v_pnum_f = 0;
  v_pmin_f = 0;
  v_pmax_f = 0;

  c_pnum_f = 0;
  c_pmin_f = 0;
  c_pmax_f = 0;
}

SVwriter::~SVwriter()
{
  // clean-up for SVwriter interface
  if (v_count != -1)
  {
    close(); // user must have forgotten to close the mesh
  }
  if (comments)
  {
    for (int i = 0; i < ncomments; i++)
    {
      free(comments[i]);
    }
    free(comments);
    ncomments = 0;
    comments = 0;
  }
  if (v_pmin_f) delete [] v_pmin_f;
  if (v_pmax_f) delete [] v_pmax_f;
  if (c_pmin_f) delete [] c_pmin_f;
  if (c_pmax_f) delete [] c_pmax_f;
}
